/********************************************************************
/*Name:		Shuang Xu
/*DePaul ID:	0255454
/*Course No:	SE455, Software Development Methods
/*Instructor:	Robert Stalzer
/*Project Name: Tetris
/*******************************************************************/

/*
Class Tetris is responsible for creating and 
displaying playing field inside an applet.
*/

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;


public class Tetris extends Applet implements Runnable  {
    	Field pf = Field.getInstance();
    	Panel scorePanel, controlPanel; 

    	Label scoreBoard = new Label( "0" );
    	Label level      = new Label( "0 " );
	Button pauseButton = new Button ("PAUSE");

      	Thread updateScoreboardThread = null;

	protected boolean paused = false;

	protected class StartButtonHandler implements ActionListener {
   		public void actionPerformed(ActionEvent event) { 
			startNewGame();       
		}
	}
	protected class PauseButtonHandler implements ActionListener {
   		public void actionPerformed(ActionEvent event) { 
			pauseGame();       
		}
	}
	protected class StopButtonHandler implements ActionListener {
   		public void actionPerformed(ActionEvent event) { 
			stopGame();       
		}
	}

/* the following GUI buttonHandlers are not used in this version
	protected class WidthChoiceHandler implements ItemListener {
   		public void itemStateChanged(ItemEvent event) { 
		       	String choice= (String)event.getItem();
		       	if ( choice.equals("14_cols") ) 	
				cols = 14;
		       	else if ( choice.equals("12_cols") ) 	
				cols = 12;
		       	else				
				cols = 10;
			pf = new PlayingField(cols, rows, color);
		       	startNewGame();
		}
	}
	protected class HeightChoiceHandler implements ItemListener {
   		public void itemStateChanged(ItemEvent event) { 
		       	String choice= (String)event.getItem();
		       	if ( choice.equals("24_rows") ) 	
				rows = 24;
		       	else if ( choice.equals("22_rows") ) 	
				rows = 22;
		       	else				
				rows = 20;
			pf = new PlayingField(cols, rows, color);
		       	startNewGame();
		}
	}
	protected class ColorChoiceHandler implements ItemListener {
   		public void itemStateChanged(ItemEvent event) { 
		       	String choice= (String)event.getItem();
		       	if ( choice.equals("white") ) 		
				color = 2;
		       	else if ( choice.equals("gray") ) 
				color = 1;
		       	else				
				color = 0;
			pf = new PlayingField(cols, rows, color);
		       	startNewGame();
		}
	}
*/
    	public void init() 	{
        	setBackground(Color.lightGray);

		setLayout(new BorderLayout());

		scorePanel = new Panel();
		scorePanel.setLayout(new GridLayout(1, 0));

        	scoreBoard.setFont( new Font( "Courier", Font.BOLD, 12 ) );
        	scoreBoard.setForeground( Color.red );
        	level.setFont( new Font( "Courier", Font.BOLD, 12 ) );
        	level.setForeground( Color.green.darker() );

		scorePanel.add( new Label( "SCORE:" ) );
		scorePanel.add( scoreBoard );
		scorePanel.add( new Label( "LEVEL:" ) );
		scorePanel.add( level );		
	    	add("North", scorePanel); 

		add("Center", pf); 

		controlPanel = new Panel(); 
		//controlPanel.setLayout(new GridLayout(2,3));    
		controlPanel.setLayout(new GridLayout(1, 0));   
		Button startButton = new Button("START");
		controlPanel.add(startButton);
		//Button pauseButton = new Button("PAUSE");
		controlPanel.add(pauseButton);     
		Button stopButton = new Button("STOP");
		controlPanel.add(stopButton);
/*
		//dropdown menu for choosing field width
		Choice fieldwidth = new Choice();
		fieldwidth.addItem("10_cols");
		fieldwidth.addItem("12_cols");
		fieldwidth.addItem("14_cols");
		controlPanel.add(fieldwidth);
		//dropdown menu for choosing field height
		Choice fieldheight = new Choice();
		fieldheight.addItem("20_rows");
		fieldheight.addItem("22_rows");
		fieldheight.addItem("24_rows");
		controlPanel.add(fieldheight);
		//dropdown menu for choosing background color
		Choice fieldcolor = new Choice();
		fieldcolor.addItem("black");
		fieldcolor.addItem("gray");
		fieldcolor.addItem("white");
		controlPanel.add(fieldcolor);
*/
		add("South", controlPanel); 

		startButton.addActionListener(new StartButtonHandler());
		startButton.addKeyListener(pf.getKeyListener());
		pauseButton.addActionListener(new PauseButtonHandler());
		pauseButton.addKeyListener(pf.getKeyListener());       	  
		stopButton.addActionListener(new StopButtonHandler()); 
/*
		fieldwidth.addItemListener(new WidthChoiceHandler());
		fieldheight.addItemListener(new HeightChoiceHandler());
		fieldcolor.addItemListener(new ColorChoiceHandler());
*/		

    }

    public void run()
    {
        while ( true ) {
            scoreBoard.setText( Integer.toString( pf.getScore() ) );
            level.setText( Integer.toString( pf.getLevel() ) );
            if ( pf.gameOver() ) {   
                displayMsg("GAME OVER!");
            }

            try {
                Thread.currentThread().sleep( 500 );
            }
            catch( InterruptedException e ) {}

        }
    }


    public void paint( Graphics g )
    {
        if ( pf.gameOver() )    
		displayMsg("GAME OVER!");
    }

 
    public void displayMsg(String msg)
    {
        Graphics g = pf.getGraphics();
        g.setFont( new Font( "Arial", Font.BOLD, 20 ) );
        FontMetrics fm = g.getFontMetrics();
        int h = pf.getSize().height, w = pf.getSize().width;
	int anchorX = (w - fm.stringWidth(msg))/2;
	int anchorY = (h - fm.getHeight())/2;
        g.setColor( Color.red.darker() );
	g.fill3DRect( anchorX, anchorY, fm.stringWidth(msg) + 4, fm.getHeight(), true );
        g.setColor( Color.black );
	g.drawString( msg, anchorX + 2, anchorY + fm.getHeight() - 5  );
        g.dispose();
    }


/*
        This method only calls startNewGame() in PlayingField.
        It is included only for purposes of being able to call
        it from JavaScript code.
*/    
    public boolean startNewGame()
    {
        pf.startNewGame();
        updateScoreboardThread = new Thread( this );
        updateScoreboardThread.start();
        return true;
    }
    public boolean pauseGame()
    {
	if (!paused) {
          pf.suspendGame();
	  pauseButton.setLabel("RESUME");
	  pauseButton.removeKeyListener(pf.getKeyListener());  
	  paused = true;
	} else {
	  pf.resumeGame();
	  pauseButton.setLabel("PAUSE");
	  pauseButton.addKeyListener(pf.getKeyListener()); 
	  paused = false;
	}
        return true;
    }
    public boolean stopGame()
    {
	if (!pf.gameOver()) displayMsg("GAME STOPPED!");
        pf.stopGame();
        return true;
    }
}

